## SHOULD PASS: <Enter description>
{{context}} "{{supported_resource}}" "ckv_unittest_pass" {
    ## Your test here
}

## SHOULD FAIL: <Enter description>
{{context}} "{{supported_resource}}" "ckv_unittest_fail" {
    ## Your test here
}
