from checkov.serverless.checks.complete import *  # noqa
from checkov.serverless.checks.custom import *  # noqa
from checkov.serverless.checks.function import *  # noqa
from checkov.serverless.checks.layer import *  # noqa
from checkov.serverless.checks.package import *  # noqa
from checkov.serverless.checks.provider import *  # noqa
from checkov.serverless.checks.service import *  # noqa